@@ -2,23 +2,39 @@ package Blosxom::Header;
 use 5.008_009;
 use strict;
 use warnings;
+use base 'Exporter';
 use constant USELESS => 'Useless use of %s with no values';
 use Blosxom::Header::Proxy;
 use Carp qw/carp/;
 use HTTP::Status qw/status_message/;
 
-our $VERSION = '0.05002';
+our $VERSION = '0.05003';
+our @EXPORT_OK = qw( $Header header_get header_set header_exists header_delete );
+
+our ( $INSTANCE, $Header );
+*Header = \$INSTANCE;
+
+sub import {
+    my ( $class, $export ) = @_;
+    my $exports_instance = $export && ( $export eq '$Header' );
+    $INSTANCE = $class->_new_instance if $exports_instance and !$INSTANCE;
+    $class->export_to_level( 1, @_ );
+}
 
 
 # Class methods
 
-our $INSTANCE;
-
 sub instance {
     my $class = shift;
-
     return $class if ref $class;
     return $INSTANCE if defined $INSTANCE;
+    $INSTANCE = $class->_new_instance;
+}
+
+sub has_instance { $INSTANCE }
+
+sub _new_instance {
+    my $class = shift;
 
     my %alias_of = (
         -content_type => '-type',
@@ -41,13 +57,15 @@ sub instance {
 
     tie my %proxy => 'Blosxom::Header::Proxy', $callback;
 
-    $INSTANCE = bless \%proxy => $class;
+    bless \%proxy => $class;
 }
 
 
 # Instance methods
 
-sub is_initialized { scalar %{ $_[0] } }
+sub is_initialized { shift->_proxy->is_initialized }
+
+sub _proxy { tied %{ $_[0] } }
 
 sub get {
     my ( $self, $field ) = @_;
@@ -93,7 +111,7 @@ sub _push {
 {
     no strict 'refs';
 
-    for my $method ( qw/attachment charset expires nph target type/ ) {
+    for my $method ( qw/attachment expires nph target/ ) {
         my $field = "-$method";
         *$method = sub {
             my $self = shift;
@@ -113,6 +131,61 @@ sub _push {
     }
 }
 
+sub type {
+    my $self    = shift;
+    my $charset = $self->{-charset};
+
+    if ( @_ ) {
+        my $type = shift;
+        delete $self->{-charset} if $charset and $type =~ /\bcharset\b/;
+        $self->{-type} = $type;
+    }
+
+    if ( my $type = $self->{-type} ) {
+        my ( $media_type, $rest ) = split /;\s*/, $type, 2;
+        $media_type =~ s/\s+//g if $media_type;
+        $media_type = lc $media_type if $media_type;
+        return wantarray ? ( $media_type, $rest ) : $media_type;
+    }
+
+    q{};
+}
+
+sub charset {
+    my $self = shift;
+    my $type = $self->{-type};
+
+    if ( @_ ) {
+        my $charset = shift;
+
+        if ( $type and $type =~ s/charset=[^;]+/charset=$charset/ ) {
+            delete $self->{-charset}; # be consistent with type()
+            $self->{-type} = $type;
+        }
+        else {
+            $self->{-charset} = $charset;
+        }
+
+        return uc $charset;
+    }
+
+    my $charset = $self->{-charset};
+
+    if ( $charset and $type and $type =~ /\bcharset\b/ ) {
+        return carp(
+            'Both of "type" and "charset" attributes specify character sets'
+        );
+    }
+    elsif ( $type and $type =~ /charset="?([^;"]+)"?/ ) {
+        return uc $1;
+    }
+    elsif ( $charset ) {
+        return uc $charset;
+    }
+
+    return;
+}
+
 sub status {
     my $self = shift;
 
@@ -121,15 +194,15 @@ sub status {
 
         if ( my $message = status_message( $code ) ) {
             $self->{-status} = "$code $message";
-            return $code;
+        }
+        else {
+            carp( qq{Unknown status code "$code" passed to status()} );
+            return;
         }
 
-        carp( qq{Unknown status code "$code" passed to status()} );
-
-        return;
+        return $code;
     }
-
-    if ( my $status = $self->{-status} ) {
+    elsif ( my $status = $self->{-status} ) {
         return substr( $status, 0, 3 );
     }
 
@@ -137,6 +210,14 @@ sub status {
 }
 
 
+# Functions to export
+
+sub header_get    { __PACKAGE__->instance->get( @_ )    }
+sub header_set    { __PACKAGE__->instance->set( @_ )    }
+sub header_exists { __PACKAGE__->instance->exists( @_ ) }
+sub header_delete { __PACKAGE__->instance->delete( @_ ) }
+
+
 # Internal functions
 
 sub _carp {
@@ -154,23 +235,43 @@ Blosxom::Header - Missing interface to modify HTTP headers
 
 =head1 SYNOPSIS
 
+  # Object-oriented interface
+
   use Blosxom::Header;
 
-  my $header = Blosxom::Header->instance;
+  my $Header = Blosxom::Header->instance;
 
-  $header->set(
+  # or
+
+  use Blosxom::Header qw/$Header/;
+
+  $Header->set(
       Status        => '304 Not Modified',
       Last_Modified => 'Wed, 23 Sep 2009 13:36:33 GMT',
   );
 
-  my $status  = $header->get( 'Status' );
-  my $bool    = $header->exists( 'ETag' );
-  my @deleted = $header->delete( qw/Content-Disposition Content-Length/ );
+  my $status = $Header->get( 'Status' );
+  my $bool = $Header->exists( 'ETag' );
+  my @deleted = $Header->delete( qw/Content-Disposition Content-Length/ );
 
-  $header->push_cookie( @cookies );
-  $header->push_p3p( @p3p );
+  $Header->push_cookie( @cookies );
+  $Header->push_p3p( @p3p );
 
-  $header->clear;
+  $Header->clear;
+
+
+  # Procedural interface
+
+  use Blosxom::Header qw/header_get header_set header_exists header_delete/;
+
+  header_set(
+      Status        => '304 Not Modified',
+      Last_Modified => 'Wed, 23 Sep 2009 13:36:33 GMT',
+  );
+
+  my $status = header_get( 'Status' );
+  my $bool = header_exists( 'ETag' );
+  my @deleted = header_delete( qw/Content-Disposition Content-Length/ );
 
 =head1 DESCRIPTION
 
@@ -216,8 +317,50 @@ into underscores in field names:
 If you follow the above normalization rule, you can modify C<$header> directly.
 In other words, this module is compatible with the way modifying C<$header>
 directly when you follow the above rule.
+L<Blosxom::Header::Fast> explains the details.
 
-=head2 METHODS
+=head2 VARIABLE
+
+The following variable is exported on demand.
+
+=over 4
+
+=item $Header
+
+The same reference as C<< Blosxom::Header->instance >> returns.
+
+  use Blosxom::Header qw/$Header/;
+
+In this case, this module creates the instance when loaded.
+Otherwise doesn't.
+
+=back
+
+=head2 FUNCTIONS
+
+The following functions are exported on demand.
+
+=over 4
+
+=item header_get()
+
+A synonym for C<< Blosxom::Header->instance->get() >>.
+
+=item header_set()
+
+A synonym for C<< Blosxom::Header->instance->set() >>.
+
+=item header_exists()
+
+A synonym for C<< Blosxom::Header->instance->exists() >>.
+
+=item header_delete()
+
+A synonym for C<< Blosxom::Header->instance->delete() >>.
+
+=back
+
+=head2 CLASS METHODS
 
 =over 4
 
@@ -225,6 +368,16 @@ directly when you follow the above rule.
 
 Returns a current Blosxom::Header object instance or create a new one.
 
+=item $header = Blosxom::Header->has_instance
+
+Returns a reference to any existing instance or C<undef> if none is defined.
+
+=back
+
+=head2 INSTANCE METHODS
+
+=over 4
+
 =item $bool = $header->is_initialized
 
 Returns a Boolean value telling whether C<$blosxom::header> is initialized or
@@ -309,13 +462,11 @@ In this case, the outgoing header will be formatted as:
 
 =item $header->charset
 
-Represents the character set sent to the browser.
+Returns the upper-cased character set specified in the Content-Type header.
 If not provided, defaults to C<ISO-8859-1>.
 
   $header->charset( 'utf-8' );
 
-NOTE: If C<< $header->type >> contains C<charset>, this attribute will be ignored.
-
 =item $header->cookie
 
 Represents the Set-Cookie headers.
@@ -384,6 +535,15 @@ If not defined, defaults to C<text/html>.
 
   $header->type( 'text/plain' );
 
+The value returned will be converted to lower case, and potential parameters
+will be chopped off and returned as a separate value if in an array context.
+If there is no such header field, then the empty string is returned.
+This makes it safe to do the following:
+
+  if ( $header->type eq 'text/html' ) {
+      ...
+  }
+
 NOTE: If this attribute isn't defined, C<CGI::header()> will add the default
 value. If you don't want to output the Content-Type header itself, you have to
 set to an empty string:
@@ -433,7 +593,7 @@ succeeded the maintenance.
 =head1 BUGS AND LIMITATIONS
 
 There are no known bugs in this module.
-Please report problems to Ryo Anazawa (anazawa@cpan.org).
+Please report problems to ANAZAWA (anazawa@cpan.org).
 Patches are welcome.
 
 =head1 AUTHOR
